#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QtWidgets/QMessageBox>
#include <QException>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect(ui->btnOpenReader,&QPushButton::clicked,this, &MainWindow::Click_OpenReader);
    connect(ui->btnCloseReader,&QPushButton::clicked,this, &MainWindow::Click_CloseReader);
    connect(ui->btnRead,&QPushButton::clicked,this, &MainWindow::Click_Read);
    connect(ui->btnWrite,&QPushButton::clicked,this, &MainWindow::Click_Write);
    connect(ui->btnInventory,&QPushButton::clicked,this, &MainWindow::Click_Inven);

    connect(ui->btnGetVer,&QPushButton::clicked,this, &MainWindow::Click_GetVer);
    connect(ui->btnGetPower,&QPushButton::clicked,this, &MainWindow::Click_GetPower);
    connect(ui->btnSetPower,&QPushButton::clicked,this, &MainWindow::Click_SetPower);
    connect(ui->btnGetRSSI,&QPushButton::clicked,this, &MainWindow::Click_GetRSSI);
    connect(ui->btnSetRSSI,&QPushButton::clicked,this, &MainWindow::Click_SetRSSI);

    connect(ui->btnWriteEPC,&QPushButton::clicked,this, &MainWindow::Click_WriteEPC);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::BeepOk()
{
    yoworfid.YW_Buzzer(1,5,5,1);
}

void MainWindow::BeepFail()
{
    yoworfid.YW_Buzzer(1,1,1,3);
}

void MainWindow::Click_OpenReader()
{
  /*  int v = yoworfid.YW_GetDLLVersion();
    QString q= QString::number(v,10);
    MsgBox(q);
*/
    int rt =yoworfid.YW_USBHIDInitial();
    if(rt>0)
    {
        MsgBox("Connect OK");

    }
    else
    {
        MsgBox("Connect Fail");
    }

}

void MainWindow::Click_CloseReader()
{
    yoworfid.YW_USBHIDFree();

}

void MainWindow::MsgBox(const QString &Content)
{
    QMessageBox::information(NULL,"YOWORFID",Content);
}

QString MainWindow::HexToStr(unsigned char *Hex, unsigned char Len)
{
    QString ret;
    for(int i=0;i<Len;i++)
    {
        QString a = "0" + QString::number(*Hex,16);
        ret = ret + a.right(2);

        Hex++;
    }

    return ret;
}

int  MainWindow::StrToHexArray(const QString &Str, unsigned char *hex)
{
    int len = Str.length()/2;
    int arraylen=0;
    for(int i=0;i<len;i++)
    {
        char H = Str.at(i*2).toLatin1();
        char L = Str.at(i*2+1).toLatin1();

        *hex = ConvertHexChar(H)*16+ConvertHexChar(L);
        hex++;
        arraylen++;
    }
    return arraylen;

}

int MainWindow::ConvertHexChar(char ch)
    {
        if((ch >= '0') && (ch <= '9'))
         return ch-'0';
        else if((ch >= 'A') && (ch <= 'F'))
         return ch-'A'+10;
        else if((ch >= 'a') && (ch <= 'f'))
         return ch-'a'+10;
        else return (-1);
    }

QString MainWindow::ByteArrToStr(QByteArray &Hex)
{
   QString ret="";
   if(Hex==nullptr)return ret;
   char b;
   foreach (b, Hex) {
        ret.append(QString("%1").arg(b,2,16,QLatin1Char('0')));
   }
   return ret;
}

QByteArrayList MainWindow::Inventory()
{
    QByteArrayList EPClist;
    int tagnums=0;
    int datalen=0;
    char epc[200];
    if(yoworfid.YW_G2_Inventory(ReaderID,true,0,0,&tagnums,&datalen,(byte*)epc)<=0)return EPClist;

    int epcindex=0;
    byte epclen=0;
    do{
    epclen =epc[epcindex];

    EPClist.append(QByteArray(&epc[epcindex+1],epclen));
    epcindex +=epclen+1;
    }while(epcindex<datalen);
    return EPClist;
}


void MainWindow::Click_Inven()
{
    QByteArrayList epclist =Inventory();
    if(epclist.count()>0)
    {

        EPCNow =epclist[0];
        ui->edtEPC->setText(ByteArrToStr(EPCNow));
        BeepOk();
    }
}

void MainWindow::Click_Read()
{
    int bank = ui->cbBank->currentIndex();
    int startaddr =ui->edtStartAddr->value();
    int bytes = ui->edtByteNums->value();
    uint password = ui->edtPassword->value();

    int datalen=0;
    byte data[200];

    if(yoworfid.YW_G2_Read(ReaderID,EPCNow.count(),(byte*)EPCNow.data(),bank,startaddr,bytes,password,&datalen,data)<=0)
    {
        MsgBox("读取失败");
        return;
    }
    ui->edtData->setText(HexToStr(data, datalen));

}

void MainWindow::Click_Write()
{
    int bank = ui->cbBank->currentIndex();
    int startaddr =ui->edtStartAddr->value();
    //int bytes = ui->edtByteNums->value();
    uint password = ui->edtPassword->value();

    int datalen=0;
    byte data[200];

    datalen = StrToHexArray(ui->edtData->text(),data);

    if(yoworfid.YW_G2_Write(ReaderID,EPCNow.count(),(byte*)EPCNow.data(),bank,startaddr,password,datalen,data)<=0)
    {
        MsgBox("写入失败");
        return;
    }
   MsgBox("写入成功");


}

void MainWindow::Click_GetVer()
{
    int v =yoworfid.YW_GetReaderVersion(ReaderID);
    ui->edtVer->setText(QString("%1.%2").arg(v/256).arg(v%256));

}
void MainWindow::Click_GetPower()
{
    int p = yoworfid.YW_GetPower(ReaderID);
   ui->edtPower->setText(QString::number(p));

}
void MainWindow::Click_SetPower()
{
    int p=1400;
    try {
        QString str = ui->edtPower->text();
        p = str.toInt();

    } catch (QException e) {
        MsgBox("数字转换失败");
        return;
    }
    if(yoworfid.YW_SetPower(ReaderID, p)<0)
    {
        MsgBox("功率设置失败");
    }
    else
    {
        MsgBox("功率设置成功");
    }

}
void MainWindow::Click_GetRSSI()
{
    int p = yoworfid.YW_GetInventoryThreshold(ReaderID);
   ui->edtRSSI->setText(QString::number(p));

}
void MainWindow::Click_SetRSSI()
{
    int p=0;
    try {
        QString str = ui->edtRSSI->text();
        p = str.toInt();

    } catch (QException e) {
        MsgBox("数字转换失败");
        return;
    }
    if(yoworfid.YW_SetInventoryThreshold(ReaderID, p)<0)
    {
        MsgBox("寻卡RSSI阈值设置失败");
    }
    else
    {
        MsgBox("寻卡RSSI阈值设置成功");
    }

}

void MainWindow::Click_WriteEPC()
{
    byte newepc[12];
    int l = StrToHexArray(ui->edtNewEPC->text().trimmed(),newepc);
    if(l%2!=0)
    {
        MsgBox("必须偶数个字节");
        return;
    }

    if(yoworfid.YW_G2_WriteEPC(ReaderID, l, newepc, 0)<0)
    {
        MsgBox("写EPC失败");
    }
    else
    {
        MsgBox("写EPC成功");
    }


}
